"""
Construction catalog window
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""


class Window:
  def __init__(self, window_id, frame_ratio, g_value, overall_u_value, name):
    self._id = window_id
    self._frame_ratio = frame_ratio
    self._g_value = g_value
    self._overall_u_value = overall_u_value
    self._name = name

  @property
  def id(self):
    """
    Get window id
    :return: str
    """
    return self._id

  @property
  def name(self):
    """
    Get window name
    :return: str
    """
    return self._name

  @property
  def frame_ratio(self):
    """
    Get window frame ratio
    :return: float
    """
    return self._frame_ratio

  @property
  def g_value(self):
    """
    Get thermal opening g-value
    :return: float
    """
    return self._g_value

  @property
  def overall_u_value(self):
    """
    Get thermal opening overall U-value in W/m2K
    :return: float
    """
    return self._overall_u_value
